#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
#define OLED_RESET -1
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

#define RIGHT_BUTTON 8
#define LEFT_BUTTON 10

#define SNAKE_SIZE 3 // Wielkość węża
#define GRID_SIZE 8 // Rozmiar kratki, w której wąż się porusza

#define SNAKE_MAX_LENGTH (SCREEN_WIDTH / GRID_SIZE * SCREEN_HEIGHT / GRID_SIZE)

int snakePos[SNAKE_MAX_LENGTH][2]; // Pozycje węża
int snakeLength = 3; // Długość węża
int foodX, foodY; // Pozycja jedzenia
int dirX = 1, dirY = 0; // Kierunek poruszania się węża
bool gameOver = false; // Czy gra się zakończyła?
bool foodEaten = true; // Czy jedzenie zostało zjedzone?

const unsigned char epd_bitmap_unnamed12 [] PROGMEM = {
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xf0, 0x03, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xc7, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0x9f, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0x9f, 0xe6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xbf, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xbf, 0xfe, 0x3f, 0xe1, 0x11, 0x06, 0x22, 0x07, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0x8f, 0xfc, 0xff, 0xc9, 0x11, 0x06, 0x22, 0x07, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xa7, 0x81, 0xff, 0xc7, 0x0b, 0xa6, 0x26, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xb7, 0x07, 0xff, 0xc3, 0x03, 0x26, 0x0e, 0x1f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xb3, 0x7f, 0xff, 0xf1, 0x23, 0x06, 0x26, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xb3, 0x7f, 0xff, 0xc9, 0x22, 0x22, 0x22, 0x07, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xb7, 0x7f, 0xff, 0xd3, 0x32, 0x22, 0x22, 0x07, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xa7, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0x8f, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xe3, 0xfc, 0x00, 0x03, 0xff, 0xe0, 0x61, 0x84, 0x20, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xc9, 0xf9, 0xfe, 0xf1, 0xff, 0xc6, 0x61, 0x85, 0x33, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xdd, 0xf3, 0xfe, 0x79, 0xff, 0xc7, 0xe9, 0x81, 0x31, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xc1, 0xf7, 0x9f, 0x00, 0xff, 0xc4, 0x41, 0x91, 0x31, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xc1, 0xf6, 0x07, 0x84, 0xff, 0xc6, 0x48, 0x93, 0x33, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xdd, 0xf0, 0xe7, 0xfd, 0xff, 0xe0, 0x88, 0x92, 0x20, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xdd, 0xf1, 0xf3, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xd0, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xc0, 0x00, 0x00, 0x00, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xcb, 0xe7, 0xff, 0x3c, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xd8, 0xcf, 0xff, 0x19, 0x9f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xdc, 0x1f, 0x07, 0xc3, 0x9f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xdf, 0xfe, 0x73, 0xff, 0x9f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xcf, 0xfc, 0xfb, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xe0, 0x00, 0x00, 0x00, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xf0, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
};

void setup() {
  pinMode(RIGHT_BUTTON, INPUT_PULLUP);
  pinMode(LEFT_BUTTON, INPUT_PULLUP);
  
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.display();
  delay(2000);
  display.clearDisplay();

    // Display the startup screen
  display.drawBitmap(0, 0, epd_bitmap_unnamed12, SCREEN_WIDTH, SCREEN_HEIGHT, WHITE);
  display.display();
  delay(2000); // Adjust delay as needed
  
  resetGame();
}

void loop() {
  if (!gameOver) {
    handleInput();
    moveSnake();
    checkCollisions();
    if (foodEaten) {
      generateFood();
      foodEaten = false;
    }
    displayGame();
    delay(100);
  } else {
    displayGameOver();
    while (digitalRead(RIGHT_BUTTON) == HIGH && digitalRead(LEFT_BUTTON) == HIGH) {
      delay(100);
    }
    resetGame();
  }
}

void resetGame() {
  snakeLength = 3;
  int startX = SCREEN_WIDTH / 2 / GRID_SIZE * GRID_SIZE;
  int startY = SCREEN_HEIGHT / 2 / GRID_SIZE * GRID_SIZE;
  for (int i = 0; i < snakeLength; i++) {
    snakePos[i][0] = startX - i * GRID_SIZE;
    snakePos[i][1] = startY;
  }
  generateFood();
  dirX = 1;
  dirY = 0;
  gameOver = false;
  delay(1000);
}

void handleInput() {
  if (digitalRead(RIGHT_BUTTON) == LOW && dirX == 0) {
    dirX = dirY;
    dirY = 0;
    dirY *= -1;
  } else if (digitalRead(LEFT_BUTTON) == LOW && dirX == 0 && (dirY == 1 || dirY == -1)) {
    dirX = dirY * -1;
    dirY = 0;
  } else if (digitalRead(RIGHT_BUTTON) == LOW && dirY == 0 && (dirX == 1 || dirX == -1)) {
    dirY = -dirX;
    dirX = 0;
  } else if (digitalRead(LEFT_BUTTON) == LOW && dirY == 0 && (dirX == 1 || dirX == -1)) {
    dirY = dirX;
    dirX = 0;
  }
}

void moveSnake() {
  for (int i = snakeLength - 1; i > 0; i--) {
    snakePos[i][0] = snakePos[i - 1][0];
    snakePos[i][1] = snakePos[i - 1][1];
  }
  snakePos[0][0] += dirX * GRID_SIZE;
  snakePos[0][1] += dirY * GRID_SIZE;
}

void checkCollisions() {
  if (snakePos[0][0] >= SCREEN_WIDTH || snakePos[0][0] < 0 || snakePos[0][1] >= SCREEN_HEIGHT || snakePos[0][1] < 0) {
    gameOver = true;
    return;
  }
  for (int i = 1; i < snakeLength; i++) {
    if (snakePos[0][0] == snakePos[i][0] && snakePos[0][1] == snakePos[i][1]) {
      gameOver = true;
      return;
    }
  }
  if (snakePos[0][0] == foodX && snakePos[0][1] == foodY) {
    snakeLength++;
    foodEaten = true;
  }
}

void generateFood() {
  foodX = random(0, SCREEN_WIDTH / GRID_SIZE) * (GRID_SIZE);
  foodY = random(0, SCREEN_HEIGHT / GRID_SIZE) * (GRID_SIZE);
}

void displayGame() {
  display.clearDisplay();
  display.drawRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, WHITE);
  display.fillRect(foodX, foodY, GRID_SIZE, GRID_SIZE, WHITE);
  for (int i = 0; i < snakeLength; i++) {
    display.fillRect(snakePos[i][0], snakePos[i][1], GRID_SIZE, GRID_SIZE, WHITE);
  }
  display.display();
}

void displayGameOver() {
  display.clearDisplay();
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(SCREEN_WIDTH / 2-60 , SCREEN_HEIGHT / 2 - 10);
  display.println("GAME OVER");
  display.setTextSize(1);
  display.display();
}
